﻿using System;
using System.Collections.Generic;
using System.ComponentModel;
using System.Data;
using System.Drawing;
using System.Linq;
using System.Text;
using System.Windows.Forms;
using Phidget22;
using Phidget22.ExampleUtils;

namespace Dictionary_Example {
	public partial class Form1 : Form {
		CommandLineOpen open;
		Phidget22.Dictionary dict = null;
		private ErrorEventBox errorBox;
		AddEntry addEntryForm;

		public Form1() {
			open = new CommandLineOpen(this);
			InitializeComponent();
		}

		public Form1(String[] commandLine) {
			open = new CommandLineOpen(this);
			open.commandLine = commandLine;
			InitializeComponent();
		}

		private void Form1_Load(object sender, EventArgs e) {
			Phidget.InvokeEventCallbacks = true;
			errorBox = new ErrorEventBox(this);

			dict = new Phidget22.Dictionary(); //create the dictionary object

			//register the various event handlers
			dict.Attach += dict_attach;
			dict.Detach += dict_detach;
			dict.Error += dict_error;
			dict.KeyValueUpdate += dict_update;
			dict.KeyAdd += dict_add;
			dict.KeyRemove += dict_remove;

			if (open.commandLine.Length == 1) { //if there is no commandline parameter then we need to create a new dictionary for test use
				Phidget22.Dictionary.AddDictionary(5000, "Test Dictionary");
				try {
					dict.DeviceSerialNumber = 5000;
					dict.Open();
				} catch (PhidgetException ex) { errorBox.addMessage("Error opening dictionary: " + ex.Message); }
			} else { //otherwise, attempt to open the specified dictionary
				try {
					open.openCmdLine(dict);
				} catch (PhidgetException ex) { errorBox.addMessage("Error connecting to dictionary server: " + ex.Message); }
			}
		}

		private void Form1_FormClosing(object sender, FormClosingEventArgs e) {
			dict.Close();
			if (dict.DeviceSerialNumber == 5000) {
				try {
					Phidget22.Dictionary.RemoveDictionary(5000);
				} catch (PhidgetException ex) { errorBox.addMessage("Error removing dictionary: " + ex.Message); }
			}

			if (addEntryForm != null)
				addEntryForm.Close();
		}

		void dict_attach(object sender, Phidget22.Events.AttachEventArgs e) {
			Phidget22.Dictionary attachedDictionary = (Phidget22.Dictionary)sender;
			dictSerialTxt.Text = attachedDictionary.DeviceSerialNumber.ToString();
			dictTxt.Text = attachedDictionary.DeviceLabel;
			dictionaryTable.Enabled = true;
			addKey.Enabled = true;
			removeKey.Enabled = true;

			//get a list of the dictionary contents
			String[] keys = dict.Scan(null); //scan returns a limited number of keys so for large dictionaries you may have to run scan multiple times.  null will start at the beginning
			while (keys.Length > 0) {

				//populate the form with the dictionary contents
				foreach (String element in keys)
					dictionaryTable.Rows.Add(element, dict.Get(element));

				keys = dict.Scan(keys[keys.Length - 1]); //now we scan beginning at the key which was the last key we previously scanned
			}
		}

		void dict_detach(object sender, Phidget22.Events.DetachEventArgs e) {
			dictSerialTxt.Clear();
			dictTxt.Clear();
			dictionaryTable.Rows.Clear();
			dictionaryTable.Enabled = false;
			addKey.Enabled = false;
			removeKey.Enabled = false;
		}

		void dict_error(object sender, Phidget22.Events.ErrorEventArgs e) {
			errorBox.addMessage(e.Description);
		}

		void dict_update(object sender, Phidget22.Events.DictionaryKeyValueUpdateEventArgs e) {
			foreach (DataGridViewRow row in dictionaryTable.Rows) {//go through the table
				if (row.Cells[0].Value.ToString() == e.Key) {//find the row with the matching key
					row.Cells[1].Value = e.Value; //update with the new value
					break; //stop looking
				}
			}
		}

		void dict_add(object sender, Phidget22.Events.DictionaryKeyAddEventArgs e) {//a new entry had been added to the dictionary, update the form to reflect this
			dictionaryTable.Rows.Add(e.Key, e.Value);
		}

		void dict_remove(object sender, Phidget22.Events.DictionaryKeyRemoveEventArgs e) {//an entry has been removed from the dictionary, update the form to reflect this
			keyRemoved(e.Key);
		}

		void keyRemoved(string key) {
			int indexToDelete = -1;
			foreach (DataGridViewRow row in dictionaryTable.Rows) {//scan the table
				if (row.Cells[0].Value.ToString() == key) {//find the row with the deleted key
					indexToDelete = row.Index;
					break;
				}
			}
			if (indexToDelete >= 0)//delete the row with the deleted key
				dictionaryTable.Rows.RemoveAt(indexToDelete);
		}

		private void addKey_Click(object sender, EventArgs e) {
			addEntryForm = new AddEntry();
			var result = addEntryForm.ShowDialog();
			if (result == DialogResult.OK) {
				try {
					dict.Add(addEntryForm.key, addEntryForm.value);
					dictionaryTable.Rows.Add(addEntryForm.key, addEntryForm.value);
				} catch (PhidgetException ex) { errorBox.addMessage("Error adding key: " + ex.Message); }
			}
		}

		private void removeKey_Click(object sender, EventArgs e) {
			foreach (DataGridViewRow selection in dictionaryTable.SelectedRows) {
				try {
					dict.Remove(selection.Cells[0].Value.ToString());
					keyRemoved(selection.Cells[0].Value.ToString());
				} catch (PhidgetException ex) { errorBox.addMessage("Error removing key: " + ex.Message); }
			}
		}
	}
}
